""" PXTL error page

Error page for template debugging. Not necessarily a great example of how to
write templates, but gets the job done.

Globals passed in:
  e - the PXTLError that was caught when executing the original page
  partial - if the template ran for a bit before dying with the optimised
    implementation, this is a string showing how much it managed to output.
    Otherwise None.
"""

# Stored as a string inside a .py module so we don't have to worry about
# storing data files in the package dir. This means we can no longer store
# the .pxc bytecode, but who cares about the performance of error pages?
#
TEMPLATE= r'''<?px :
  import os, string, urllib, traceback, linecache
  from pxtl.constants import *

  # Number of rows of template text to show above and below the error line in
  # the 'source node' section.
  #
  SHOWROWS= 5

  # Utility functions to extract scope info.
  #
  def visibleItems(scope):
    d= {}
    for k, v in scope.items():
      if k[:2]!='__':
        d[k]= v
    return d
  def getClassMembers(mems, c):
    for base in c.__bases__:
      getClassMembers(mems, base)
    for key in dir(c):
      mems[key]= getattr(c, key)
?>

<html
  xmlns="http://www.w3.org/1999/xhtml"
  xmlns:px="http://www.doxdesk.com/pxtl"
  px:doctype="pxtl.XHTML1S"
>
<head>
  <title> <?_ e.__class__ ?> </title>
  <style type="text/css"><![CDATA[
    body {
      margin: 0; padding: 16px;
      font-family: "Palatino Linotype", Palatino, Sylfaen, "Times New Roman", serif;
    }

    h1 { font-style: normal; }
    h2 { font-style: italic; }
    h1, h2, div.end {
      font-size: 100%; font-weight: bold;
      border-bottom: dashed red 1px;
    }

    table {
      table-layout: fixed; width: 100%;
      border-collapse: collapse;
    }
    tr { vertical-align: top; }
    td, th { text-align: left; padding: 0; }
    tr .narrow { width: 4em; }
    tr .wide { width: 8em; }

    .folder {
      border: solid rgb(232, 232, 232) 2px;
      padding: 0 2px; margin: 0 1px;
      font-size: 75%;
    }
    a.folder { color: black; text-decoration: none; }
    span.folder { color: grey; }

    tr.trace, span.margin { background-color: rgb(232, 232, 232); }
    span.badchar { border: solid red 1px; }
    span.marker { color: red; }

  ]]></style>
</head>
<body>

  <h1> Error running PXTL template </h1>
  <p> <?_ e ?> </p>

  <px:if test="hasattr(e, 'trace')">
    <h2> Traceback </h2>
    <?px trace= e.trace.tb_next ?>
    <table>
      <thead><tr>
        <th class="narrow"> Vars </th>
        <th class="wide"> Function </th>
        <th class="wide"> File </th>
        <th class="narrow"> Line </th>
        <th> Code </th>
      </tr></thead>
      <px:while test="trace is not None" index="frameno">

        <?px :
          frame= trace.tb_frame
          line= traceback.tb_lineno(trace)
          file= frame.f_code.co_filename
          trace= trace.tb_next
          vg= visibleItems(frame.f_globals)
          vl= {}
          vm= {}
          if frame.f_locals is not frame.f_globals:
            vl= visibleItems(frame.f_locals)
            if frame.f_locals.has_key('self'):
              self= frame.f_locals['self']
              try:
                if hasattr(self, '__class__'):
                  mems= {}
                  getClassMembers(mems, self.__class__)
                  for key in dir(self):
                    mems[key]= getattr(self, key)
                  vm= visibleItems(mems)
                  del vl['self']
              except Exception:
                pass
          vars= [('globals', vg), ('locals', vl), ('members', vm)]
        ?>
        <tbody><tr class="trace">

          <?__ Output var-showing buttons ?>
          <td>
            <px:for item="vartype, varvals" in="vars" px:space="0">
              <px:if test="len(varvals)>0">
                <a class="folder" href="#{?_ vartype ?}{?_ frameno ?}" title="Show {?_ vartype ?}">
                  <?_ string.upper(vartype[0]) ?>
                </a>
              </px:if>
              <px:else>
                <span class="folder" title="No {?_ vartype ?}">
                  <?_ string.upper(vartype[0]) ?>
                </span>
              </px:else>
            </px:for>
          </td>

          <?__ Output rest of traceback line ?>
          <td><?_ frame.f_code.co_name ?></td>
          <td title="{?_ file ?}">
            <px:if test="file is not None and file[:1]!='&lt;'"><?_ os.path.basename(file) ?></px:if>
            <px:else>-</px:else>
          </td>
          <td><?_ line ?></td>
          <td><code>
            <px:if test="file is not None and file[:1]!='&lt;'">
              <?_ linecache.getline(file, line) ?>
            </px:if>
          </code></td>
        </tr></tbody>

        <?__ List each scopeful of variables ?>
        <px:for item="vartype, varvals" in="vars"><px:if test="len(varvals)>0">
          <?px :
            varitems= varvals.items()
            varitems.sort()
          ?>
          <tbody class="foldable" id="{?_ vartype ?}{?_ frameno ?}">
            <px:for item="key, value" in="varitems" index="vix">
              <?px :
                try:
                  reprvalue= repr(value)
                except Exception:
                  reprvalue= type(value)
              ?>
              <tr>
                <td/>
                <td><px:if test="vix==0"><code><strong><?_ vartype ?></strong></code></px:if></td>
                <td><code><em><?_ key ?></em></code></td>
                <td colspan="2"><code><?_ reprvalue ?></code></td>
              </tr>
            </px:for>
          </tbody>
        </px:if></px:for>
      </px:while>

      <?__ Include extra row showing bad code in case of SyntaxError ?>
      <px:if test="isinstance(e.error, SyntaxError)">
        <tbody><tr class="trace">
          <td/><td/>
          <td title="{?_ e.error.filename ?}">
            <px:if test="e.error.filename is not None"><?_ os.path.basename(e.error.filename) ?></px:if>
          </td>
          <td><?_ e.error.lineno ?></td>
          <td><code px:space="0">
            <px:if test="e.error.offset is not None">
              <?_ e.error.text[:e.error.offset] ?>
              <span class="badchar"><?_ e.error.text[e.error.offset:e.error.offset+1] ?></span>
              <?_ e.error.text[e.error.offset+1:] ?>
            </px:if>
            <px:else>poo <?_ e.error.text ?></px:else>
          </code></td>
        </tr></tbody>
      </px:if>
    </table>
  </px:if>


  <?__ If the DOM in use was pxdom, we can find out where in the original
       XML template the error occured. Display the content near that code
       and highlight the dodgy code. Note: non-UTF-encoded docs without a
       protocol-level charset will be read with the wrong encoding which might
       put the line/column position in the wrong place. We don't really want
       to go trying to parse the XML declaration again at this late stage. Use
       UTF encodings! They are good!
  ?>
  <px:if test="hasattr(e.context, 'pxdomLocation') and e.context.pxdomLocation is not None">
    <?px :
      documentURI= e.context.pxdomLocation.uri
      if documentURI:
        from pxtl import pxdom
        inp= pxdom.getDOMImplementation('').createLSInput()
        inp.systemId= documentURI
        buf= pxdom.InputBuffer(inp, (1, 1), pxdom.DOMConfiguration(), True)
        template= string.split(buf.chars, '\n')
        row, col= e.context.pxdomLocation.lineNumber-1, e.context.pxdomLocation.columnNumber-1

        prerows= template[max(row-SHOWROWS, 0):row]
        therow= (template[row:row+1]+[''])[0]
        postrows= template[row+1:row+1+SHOWROWS]
    ?>
    <px:if test="documentURI">
      <h2> Source node </h2>
      <pre px:space="0">
        <px:for item="row" in="prerows" index="rowix">
          <span class="margin">|</span>
          <?_ row ?><?_ '\n' ?>
        </px:for>
        <span class="margin">
          <span class="marker">*</span>
          <?_ therow[:col] ?>
          <span class="badchar"><?_ therow[col:col+1] ?></span>
          <?_ therow[col+1:] ?><?_ '\n' ?>
        </span>
        <px:for item="row" in="postrows" index="rowix">
          <span class="margin">|</span>
          <?_ row ?><?_ '\n' ?>
        </px:for>
      </pre>
    </px:if>
  </px:if>

  <px:if test="partial">
    <h2> Partial output </h2>
    <pre id="partial"><?_ partial ?></pre>
  </px:if>

  <div class="end">&#160;</div>

  <?__ Folding script. If JS is enabled, all scope rows will disappear from
       the traceback table. They can then be toggled on and off with the
       link buttons. If JS is not available they will stay in the document,
       linked from the buttons.

       Note IE/Win cannot set an element's display property to 'table-...'
       values, it uses 'block' to do the same thing. We will use a dodgy
       JScript conditional-comment hack to detect this.
  ?>
  <script type="text/javascript"><![CDATA[

    TABLE_ROW= 'table-row-group';
    /*@cc_on
    @if (@_win32)
      TABLE_ROW= 'block'
    @end @*/

    function fold(e) {
      var row= document.getElementById(this.hash.substring(1));
      var display= (row.style.display=='none');
      row.style.display= display ? TABLE_ROW : 'none';
      this.style.borderStyle= display ? 'inset' : 'outset';
      return false;
    }

    if (document.getElementsByTagName) {
      var links= document.getElementsByTagName('a');
      for (var i= links.length-1; i!=-1; i=i-1)
        if (links[i].className=='folder') {
          links[i].onclick= fold;
          links[i].ondblclick= fold;
          links[i].style.borderStyle= 'outset';
          links[i].style.borderColor= 'white';
        }
      var rows= document.getElementsByTagName('tbody');
      for (var i= rows.length-1; i!=-1; i=i-1)
        if (rows[i].className=='foldable')
          rows[i].style.display= 'none';
    }
  ]]></script>
</body></html>'''
